#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/9/24 15:00
Desc: 东方财富-港股-行业对比
https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
"""

import pandas as pd
import requests


def stock_hk_growth_comparison_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-行业对比-成长性对比
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
    :param symbol: 股票代码
    :type symbol: str
    :return: 成长性对比
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_PCF10_INDUSTRY_HKGROWTH',
        'columns': 'SECUCODE,SECURITY_CODE,ORG_CODE,REPORT_DATE,TYPE_ID,TYPE_TYPE,'
                   'TYPE_NAME,TYPE_NAME_EN,CORRE_SECURITY_CODE,CORRE_SECUCODE,'
                   'CORRE_SECURITY_NAME,EPS_YOY,OPERATE_INCOME_YOY,OPERATE_PROFIT_YOY,'
                   'TOTAL_ASSET_YOY,EPS_YOY_RANK,OPINCOME_YOY_RANK,OPROFIT_YOY_RANK,TOASSET_YOY_RANK',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol}.HK")(CORRE_SECUCODE="{symbol}.HK")',
        'pageNumber': '1',
        'pageSize': '',
        'sortTypes': '',
        'sortColumns': '',
        'source': 'F10',
        'client': 'PC',
        'v': '03313416193688571'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    field_mapping = {
        'CORRE_SECURITY_CODE': '代码',
        'CORRE_SECURITY_NAME': '简称',
        'EPS_YOY': '基本每股收益同比增长率',
        'EPS_YOY_RANK': '基本每股收益同比增长率排名',
        'OPERATE_INCOME_YOY': '营业收入同比增长率',
        'OPINCOME_YOY_RANK': '营业收入同比增长率排名',
        'OPERATE_PROFIT_YOY': '营业利润率同比增长率',
        'OPROFIT_YOY_RANK': '营业利润率同比增长率排名',
        'TOTAL_ASSET_YOY': '基本每股收总资产同比增长率益同比增长率',
        'TOASSET_YOY_RANK': '总资产同比增长率排名',
    }
    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json['result'] is not None:
        temp_df = pd.DataFrame(data_json['result']['data'])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]
    return temp_df


def stock_hk_valuation_comparison_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-行业对比-估值对比
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
    :param symbol: 股票代码
    :type symbol: str
    :return: 估值对比
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_PCF10_INDUSTRY_HKCVALUE',
        'columns': 'SECUCODE,SECURITY_CODE,ORG_CODE,REPORT_DATE,TYPE_ID,'
                   'TYPE_TYPE,TYPE_NAME,TYPE_NAME_EN,CORRE_SECURITY_CODE,'
                   'CORRE_SECUCODE,CORRE_SECURITY_NAME,PE_TTM,PE_LYR,PB_MQR,'
                   'PB_LYR,PS_TTM,PS_LYR,PCE_TTM,PCE_LYR,PE_TTM_RANK,PE_LYR_RANK,'
                   'PB_MQR_RANK,PB_LYR_RANK,PS_TTM_RANK,PS_LYR_RANK,PCE_TTM_RANK,PCE_LYR_RANK',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol}.HK")(CORRE_SECUCODE="{symbol}.HK")',
        'pageNumber': '1',
        'pageSize': '',
        'sortTypes': '',
        'sortColumns': '',
        'source': 'F10',
        'client': 'PC',
        'v': '03445297742754925'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    field_mapping = {
        'CORRE_SECURITY_CODE': '代码',
        'CORRE_SECURITY_NAME': '简称',
        'PE_TTM': '市盈率-TTM',
        'PE_TTM_RANK': '市盈率-TTM排名',
        'PE_LYR': '市盈率-LYR',
        'PE_LYR_RANK': '市盈率-LYR排名',
        'PB_MQR': '市净率-MRQ',
        'PB_MQR_RANK': '市净率-MRQ排名',
        'PB_LYR': '市净率-LYR',
        'PB_LYR_RANK': '市净率-LYR排名',
        'PS_TTM': '市销率-TTM',
        'PS_TTM_RANK': '市销率-TTM排名',
        'PS_LYR': '市销率-LYR',
        'PS_LYR_RANK': '市销率-LYR排名',
        'PCE_TTM': '市现率-TTM',
        'PCE_TTM_RANK': '市现率-TTM排名',
        'PCE_LYR': '市现率-LYR',
        'PCE_LYR_RANK': '市现率-LYR排名',
    }
    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json['result'] is not None:
        temp_df = pd.DataFrame(data_json['result']['data'])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]
    return temp_df


def stock_hk_scale_comparison_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-行业对比-规模对比
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/IndustryComparison
    :param symbol: 股票代码
    :type symbol: str
    :return: 规模对比
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_PCF10_INDUSTRY_SCALE',
        'columns': 'SECURITY_CODE,SECUCODE,TYPE_ID,TYPE_TYPE,TYPE_NAME,'
                   'TYPE_NAME_EN,CORRE_SECURITY_CODE,CORRE_SECUCODE,'
                   'CORRE_SECURITY_NAME,MAXSTDREPORTDATE,HKSDQMV,'
                   'HKTOTAL_MARKET_CAP,OPERATE_INCOME,GROSS_PROFIT,'
                   'HKSDQMV_RANK,HKTOTAL_CAP_RANK,OPERATE_INCOME_RANK,GROSS_PROFIT_RANK',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol}.HK")(CORRE_SECUCODE="{symbol}.HK")',
        'pageNumber': '1',
        'pageSize': '',
        'sortTypes': '',
        'sortColumns': '',
        'source': 'F10',
        'client': 'PC',
        'v': '07839693368708753'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    field_mapping = {
        'CORRE_SECURITY_CODE': '代码',
        'CORRE_SECURITY_NAME': '简称',
        'HKSDQMV': '总市值',
        'HKSDQMV_RANK': '总市值排名',
        'HKTOTAL_MARKET_CAP': '流通市值',
        'HKTOTAL_CAP_RANK': '流通市值排名',
        'OPERATE_INCOME': '营业总收入',
        'OPERATE_INCOME_RANK': '营业总收入排名',
        'GROSS_PROFIT': '净利润',
        'GROSS_PROFIT_RANK': '净利润排名',
    }
    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json['result'] is not None:
        temp_df = pd.DataFrame(data_json['result']['data'])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]
    return temp_df


if __name__ == "__main__":
    stock_hk_growth_comparison_em_df = stock_hk_growth_comparison_em(symbol="03900")
    print(stock_hk_growth_comparison_em_df)

    stock_hk_valuation_comparison_em_df = stock_hk_valuation_comparison_em(symbol="03900")
    print(stock_hk_valuation_comparison_em_df)

    stock_hk_scale_comparison_em_df = stock_hk_scale_comparison_em(symbol="03900")
    print(stock_hk_scale_comparison_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/10/30 17:00
Desc: 查询期货合约当前时刻的详情
https://finance.sina.com.cn/futures/quotes/V2101.shtml
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup


def futures_contract_detail(symbol: str = "AP2101") -> pd.DataFrame:
    """
    查询期货合约详情
    https://finance.sina.com.cn/futures/quotes/V2101.shtml
    :param symbol: 合约
    :type symbol: str
    :return: 期货合约详情
    :rtype: pandas.DataFrame
    """
    url = f"https://finance.sina.com.cn/futures/quotes/{symbol}.shtml"
    r = requests.get(url)
    r.encoding = "gb2312"
    temp_df = pd.read_html(StringIO(r.text))[6]
    data_one = temp_df.iloc[:, :2]
    data_one.columns = ["item", "value"]
    data_two = temp_df.iloc[:, 2:4]
    data_two.columns = ["item", "value"]
    data_three = temp_df.iloc[:, 4:]
    data_three.columns = ["item", "value"]
    temp_df = pd.concat(
        objs=[data_one, data_two, data_three], axis=0, ignore_index=True
    )
    return temp_df


def futures_contract_detail_em(symbol: str = "v2602F") -> pd.DataFrame:
    """
    查询期货合约详情
    https://quote.eastmoney.com/qihuo/v2602F.html
    :param symbol: 合约
    :type symbol: str
    :return: 期货合约详情
    :rtype: pandas.DataFrame
    """
    url = f"https://quote.eastmoney.com/qihuo/{symbol}.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    url_text = \
        soup.find(name="div", attrs={"class": "sidertabbox_tsplit"}).find(name="div", attrs={"class": "onet"}).find(
            "a")[
            'href']
    inner_symbol = url_text.split("#")[-1].strip("futures_")
    url = f"https://futsse-static.eastmoney.com/redis?msgid={inner_symbol}_info"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.DataFrame.from_dict(data_json, orient="index")
    column_mapping = {
        'vname': '交易品种',
        'vcode': '交易代码',
        'jydw': '交易单位',
        'bjdw': '报价单位',
        'market': '上市交易所',
        'zxbddw': '最小变动价格',
        'zdtbfd': '跌涨停板幅度',
        'hyjgyf': '合约交割月份',
        'jysj': '交易时间',
        'zhjyr': '最后交易日',
        'zhjgr': '最后交割日',
        'jgpj': '交割品级',
        'zcjybzj': '最初交易保证金',
        'jgfs': '交割方式'
    }
    temp_df.rename(index=column_mapping, inplace=True)
    temp_df.reset_index(drop=False, inplace=True)
    temp_df.columns = ['item', 'value']
    return temp_df


if __name__ == "__main__":
    futures_contract_detail_df = futures_contract_detail(symbol="V2101")
    print(futures_contract_detail_df)

    futures_contract_detail_em_df = futures_contract_detail_em(symbol="l2602F")
    print(futures_contract_detail_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/21 18:00
Desc: 科创板报告
https://data.eastmoney.com/notices/kcb.html
"""

import pandas as pd
import requests
from tqdm import tqdm


def _stock_zh_kcb_report_em_page() -> int:
    """
    科创板报告的页数
    https://data.eastmoney.com/notices/kcb.html
    :return: 科创板报告的页数
    :rtype: int
    """
    url = "https://np-anotice-stock.eastmoney.com/api/security/ann"
    params = {
        "sr": "-1",
        "page_size": "100",
        "page_index": "1",
        "ann_type": "KCB",
        "client_source": "web",
        "f_node": "0",
        "s_node": "0",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = int(
        int(data_json["data"]["total_hits"]) / int(data_json["data"]["page_size"])
    )
    return page_num


def stock_zh_kcb_report_em(from_page: int = 1, to_page: int = 100) -> pd.DataFrame:
    """
    科创板报告内容
    https://data.eastmoney.com/notices/kcb.html
    :param from_page: 开始获取的页码
    :type from_page: int
    :param to_page: 结束获取的页码
    :type to_page: int
    :return: 科创板报告内容
    :rtype: pandas.DataFrame
    """
    url = "https://np-anotice-stock.eastmoney.com/api/security/ann"
    total_page = _stock_zh_kcb_report_em_page()
    big_df = pd.DataFrame()
    if to_page >= total_page:
        to_page = total_page
    for i in tqdm(range(from_page, to_page + 1), leave=False):
        params = {
            "sr": "-1",
            "page_size": "100",
            "page_index": i,
            "ann_type": "KCB",
            "client_source": "web",
            "f_node": "0",
            "s_node": "0",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [
                [item["codes"][0]["stock_code"] for item in data_json["data"]["list"]],
                [item["codes"][0]["short_name"] for item in data_json["data"]["list"]],
                [item["title"] for item in data_json["data"]["list"]],
                [
                    item["columns"][0]["column_name"]
                    for item in data_json["data"]["list"]
                ],
                [item["notice_date"] for item in data_json["data"]["list"]],
                [item["art_code"] for item in data_json["data"]["list"]],
            ]
        ).T
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.columns = [
        "代码",
        "名称",
        "公告标题",
        "公告类型",
        "公告日期",
        "公告代码",
    ]
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


if __name__ == "__main__":
    stock_zh_kcb_report_em_df = stock_zh_kcb_report_em(from_page=1, to_page=100)
    print(stock_zh_kcb_report_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/7 15:00
Desc: 东方财富网-数据中心-研究报告-盈利预测
https://data.eastmoney.com/report/profitforecast.jshtml
"""

from io import StringIO

import pandas as pd
import requests


def stock_hk_profit_forecast_et(
    symbol: str = "09999", indicator: str = "盈利预测概览"
) -> pd.DataFrame:
    """
    经济通-公司资料-盈利预测
    https://www.etnet.com.hk/www/sc/stocks/realtime/quote_profit.php?code=9999
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: "盈利预测概览"; choice of {"评级总览", "去年度业绩表现", "综合盈利预测", "盈利预测概览"}
    :type indicator: str
    :return: 盈利预测
    :rtype: pandas.DataFrame
    """
    url = "https://www.etnet.com.hk/www/sc/stocks/realtime/quote_profit.php"
    params = {
        "code": str(int(symbol)),
    }
    r = requests.get(url, params=params)
    if indicator == "评级总览":
        temp_df = pd.read_html(StringIO(r.text))[0]
        inner_list = [item for item in temp_df.iloc[0, 0].split(" ") if item != ""]
        inner_list.remove("平均评级")
        temp_df = pd.DataFrame(inner_list).T
        temp_df.columns = ["方向", "评级数量", "平均评级"]
        return temp_df
    elif indicator == "去年度业绩表现":
        temp_df = pd.read_html(StringIO(r.text))[2]
        temp_df_upper = temp_df.iloc[:, :2].copy()
        temp_df_upper.reset_index(inplace=True, drop=True)
        temp_df_upper.columns = ["item", "value"]
        temp_df_down = temp_df.iloc[:, 3:].copy()
        temp_df_down.reset_index(inplace=True, drop=True)
        temp_df_down.columns = ["item", "value"]
        temp_df = pd.concat(objs=[temp_df_upper, temp_df_down], ignore_index=True)
        return temp_df
    elif indicator == "综合盈利预测":
        temp_df = pd.read_html(StringIO(r.text), header=0)[3]
        temp_df.rename(
            columns={
                "纯利/(亏损)  (百万元人民币)": "纯利/亏损",
                "纯利/(亏损)  (百万港元)": "纯利/亏损",
                "每股盈利/  (亏损)(分)": "每股盈利/每股亏损",
                "每股盈利/  (亏损)(港仙)": "每股盈利/每股亏损",
                "每股派息  (分)": "每股派息",
                "每股派息  (港仙)": "每股派息",
                "每股资产净值  (人民币元)": "每股资产净值",
                "每股资产净值  (港元)": "每股资产净值",
                "最高  (百万元人民币)": "最高",
                "最高  (百万港元)": "最高",
                "最低  (百万元人民币)": "最低",
                "最低  (百万港元)": "最低",
            },
            inplace=True,
        )
        temp_df["纯利/亏损"] = pd.to_numeric(temp_df["纯利/亏损"], errors="coerce")
        temp_df["每股盈利/每股亏损"] = pd.to_numeric(
            temp_df["每股盈利/每股亏损"], errors="coerce"
        )
        temp_df["每股派息"] = pd.to_numeric(temp_df["每股派息"], errors="coerce")
        temp_df["每股资产净值"] = pd.to_numeric(
            temp_df["每股资产净值"], errors="coerce"
        )
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        return temp_df
    elif indicator == "盈利预测概览":
        temp_df = pd.read_html(StringIO(r.text), header=0)[4]
        del temp_df["目标价* (港元).1"]
        temp_df.rename(
            columns={
                "纯利/(亏损)  (百万元人民币)": "纯利/亏损",
                "纯利/(亏损)  (百万港元)": "纯利/亏损",
                "每股盈利*/ (亏损)  (港仙)": "每股盈利",
                "每股盈利*/ (亏损)  (分)": "每股盈利",
                "每股派息*  (分)": "每股派息",
                "每股派息*  (港仙)": "每股派息",
                "目标价* (港元)": "目标价",
            },
            inplace=True,
        )
        temp_df.dropna(inplace=True)
        temp_df["纯利/亏损"] = pd.to_numeric(temp_df["纯利/亏损"], errors="coerce")
        temp_df["每股盈利"] = pd.to_numeric(temp_df["每股盈利"], errors="coerce")
        temp_df["每股派息"] = pd.to_numeric(temp_df["每股派息"], errors="coerce")
        temp_df["目标价"] = pd.to_numeric(temp_df["目标价"], errors="coerce")
        temp_df["更新日期"] = pd.to_datetime(
            temp_df["更新日期"], errors="coerce", dayfirst=True
        ).dt.date
        temp_df["财政年度"] = temp_df["财政年度"].astype(int).astype(str)
        return temp_df


if __name__ == "__main__":
    stock_hk_profit_forecast_et_df = stock_hk_profit_forecast_et(
        symbol="09999", indicator="评级总览"
    )
    print(stock_hk_profit_forecast_et_df)

    stock_hk_profit_forecast_et_df = stock_hk_profit_forecast_et(
        symbol="09999", indicator="去年度业绩表现"
    )
    print(stock_hk_profit_forecast_et_df)

    stock_hk_profit_forecast_et_df = stock_hk_profit_forecast_et(
        symbol="09999", indicator="综合盈利预测"
    )
    print(stock_hk_profit_forecast_et_df)

    stock_hk_profit_forecast_et_df = stock_hk_profit_forecast_et(
        symbol="09999", indicator="盈利预测概览"
    )
    print(stock_hk_profit_forecast_et_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/9/8 16:20
Desc: 上海证券交易所-产品-股票期权-信息披露-当日合约
http://www.sse.com.cn/assortment/options/disclo/preinfo/
"""

import pandas as pd
import requests


def option_current_day_sse() -> pd.DataFrame:
    """
    上海证券交易所-产品-股票期权-信息披露-当日合约
    http://www.sse.com.cn/assortment/options/disclo/preinfo/
    :return: 上交所期权当日合约
    :rtype: pandas.DataFrame
    """
    url = "http://query.sse.com.cn/commonQuery.do"
    params = {
        "isPagination": "false",
        "expireDate": "",
        "securityId": "",
        "sqlId": "SSE_ZQPZ_YSP_GGQQZSXT_XXPL_DRHY_SEARCH_L",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "query.sse.com.cn",
        "Pragma": "no-cache",
        "Referer": "http://www.sse.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/101.0.4951.67 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"])
    dict_df = {
        "SECURITY_ID": "合约编码",
        "CONTRACT_ID": "合约交易代码",
        "CONTRACT_SYMBOL": "合约简称",
        "SECURITYNAMEBYID": "标的券名称及代码",
        "CALL_OR_PUT": "类型",
        "EXERCISE_PRICE": "行权价",
        "CONTRACT_UNIT": "合约单位",
        "END_DATE": "期权行权日",
        "DELIVERY_DATE": "行权交收日",
        "EXPIRE_DATE": "到期日",
        "START_DATE": "开始日期",
    }
    temp_df = temp_df[dict_df.keys()].rename(columns=dict_df)
    return temp_df


if __name__ == "__main__":
    option_current_day_sse_df = option_current_day_sse()
    print(option_current_day_sse_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/6/17 14:00
Desc: 股票指数成份股数据, 新浪有两个接口, 这里使用老接口:
新接口：https://vip.stock.finance.sina.com.cn/mkt/#zhishu_000001
老接口：https://vip.stock.finance.sina.com.cn/corp/view/vII_NewestComponent.php?page=1&indexid=399639
"""

import math
from io import BytesIO, StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup

from akshare.utils import demjson


def index_stock_cons_sina(symbol: str = "000300") -> pd.DataFrame:
    """
    新浪新版股票指数成份页面, 目前该接口可获取指数数量较少
    https://vip.stock.finance.sina.com.cn/mkt/#zhishu_000040
    :param symbol: 指数代码
    :type symbol: str
    :return: 指数的成份股
    :rtype: pandas.DataFrame
    """
    if symbol == "000300":
        symbol = "hs300"
        url = (
            "https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php"
            "/Market_Center.getHQNodeStockCountSimple"
        )
        params = {"node": f"{symbol}"}
        r = requests.get(url, params=params)
        page_num = math.ceil(int(r.json()) / 80) + 1
        temp_df = pd.DataFrame()
        for page in range(1, page_num):
            url = "https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData"
            params = {
                "page": str(page),
                "num": "80",
                "sort": "symbol",
                "asc": "1",
                "node": "hs300",
                "symbol": "",
                "_s_r_a": "init",
            }
            r = requests.get(url, params=params)
            temp_df = pd.concat(
                objs=[temp_df, pd.DataFrame(demjson.decode(r.text))], ignore_index=True
            )
        return temp_df

    url = "https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeDataSimple"
    params = {
        "page": 1,
        "num": "3000",
        "sort": "symbol",
        "asc": "1",
        "node": f"zhishu_{symbol}",
        "_s_r_a": "setlen",
    }
    r = requests.get(url, params=params)
    temp = pd.DataFrame(demjson.decode(r.text))
    return temp


def index_stock_info() -> pd.DataFrame:
    """
    聚宽-指数数据-指数列表
    https://www.joinquant.com/data/dict/indexData
    :return: 指数信息的数据框
    :rtype: pandas.DataFrame
    """
    url = "https://www.joinquant.com/data/dict/indexData"
    r = requests.get(url)
    r.encoding = "utf-8"
    index_df = pd.read_html(StringIO(r.text))[0]
    index_df["指数代码"] = index_df["指数代码"].str.split(".", expand=True)[0]
    index_df.columns = ["index_code", "display_name", "publish_date", "-", "-"]
    temp_df = index_df[["index_code", "display_name", "publish_date"]].copy()
    return temp_df


def index_stock_cons(symbol: str = "399639") -> pd.DataFrame:
    """
    最新股票指数的成份股目录
    https://vip.stock.finance.sina.com.cn/corp/view/vII_NewestComponent.php?page=1&indexid=399639
    :param symbol: 指数代码, 可以通过 ak.index_stock_info() 函数获取
    :type symbol: str
    :return: 最新股票指数的成份股目录
    :rtype: pandas.DataFrame
    """
    url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vII_NewestComponent/indexid/{symbol}.phtml"
    r = requests.get(url)
    r.encoding = "gb2312"
    soup = BeautifulSoup(r.text, "lxml")
    page_num = (
        soup.find(attrs={"class": "table2"})
        .find("td")
        .find_all("a")[-1]["href"]
        .split("page=")[-1]
        .split("&")[0]
    )
    if page_num == "#":
        temp_df = pd.read_html(StringIO(r.text), header=0, skiprows=1)[3].iloc[:, :3]
        temp_df["品种代码"] = temp_df["品种代码"].astype(str).str.zfill(6)
        return temp_df

    temp_df = pd.DataFrame()
    for page in range(1, int(page_num) + 1):
        url = f"https://vip.stock.finance.sina.com.cn/corp/view/vII_NewestComponent.php?page={page}&indexid={symbol}"
        r = requests.get(url)
        r.encoding = "gb2312"
        temp_df = pd.concat(
            objs=[temp_df, pd.read_html(StringIO(r.text), header=1)[3]],
            ignore_index=True,
        )
    temp_df = temp_df.iloc[:, :3]
    temp_df["品种代码"] = temp_df["品种代码"].astype(str).str.zfill(6)
    return temp_df


def index_stock_cons_csindex(symbol: str = "000300") -> pd.DataFrame:
    """
    中证指数网站-成份股目录
    https://www.csindex.com.cn/zh-CN/indices/index-detail/000300
    :param symbol: 指数代码, 可以通过 ak.index_stock_info() 函数获取
    :type symbol: str
    :return: 最新指数的成份股
    :rtype: pandas.DataFrame
    """
    url = (
        f"https://oss-ch.csindex.com.cn/static/"
        f"html/csindex/public/uploads/file/autofile/cons/{symbol}cons.xls"
    )
    r = requests.get(url)
    temp_df = pd.read_excel(BytesIO(r.content))
    temp_df.columns = [
        "日期",
        "指数代码",
        "指数名称",
        "指数英文名称",
        "成分券代码",
        "成分券名称",
        "成分券英文名称",
        "交易所",
        "交易所英文名称",
    ]
    temp_df["日期"] = pd.to_datetime(
        temp_df["日期"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["指数代码"] = temp_df["指数代码"].astype(str).str.zfill(6)
    temp_df["成分券代码"] = temp_df["成分券代码"].astype(str).str.zfill(6)
    return temp_df


def index_stock_cons_weight_csindex(symbol: str = "000300") -> pd.DataFrame:
    """
    中证指数网站-样本权重
    https://www.csindex.com.cn/zh-CN/indices/index-detail/000300
    :param symbol: 指数代码, 可以通过 ak.index_stock_info() 接口获取
    :type symbol: str
    :return: 最新指数的成份股权重
    :rtype: pandas.DataFrame
    """
    url = (
        f"https://oss-ch.csindex.com.cn/static/html/csindex/"
        f"public/uploads/file/autofile/closeweight/{symbol}closeweight.xls"
    )
    r = requests.get(url)
    temp_df = pd.read_excel(BytesIO(r.content))
    temp_df.columns = [
        "日期",
        "指数代码",
        "指数名称",
        "指数英文名称",
        "成分券代码",
        "成分券名称",
        "成分券英文名称",
        "交易所",
        "交易所英文名称",
        "权重",
    ]
    temp_df["日期"] = pd.to_datetime(
        temp_df["日期"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["指数代码"] = temp_df["指数代码"].astype(str).str.zfill(6)
    temp_df["成分券代码"] = temp_df["成分券代码"].astype(str).str.zfill(6)
    temp_df["权重"] = pd.to_numeric(temp_df["权重"], errors="coerce")
    return temp_df


def stock_a_code_to_symbol(symbol: str = "000300") -> str:
    """
    输入股票代码判断股票市场
    :param symbol: 股票代码
    :type symbol: str
    :return: 股票市场
    :rtype: str
    """
    if symbol.startswith("6") or symbol.startswith("900"):
        return f"sh{symbol}"
    else:
        return f"sz{symbol}"


if __name__ == "__main__":
    index_stock_cons_csindex_df = index_stock_cons_csindex(symbol="000300")
    print(index_stock_cons_csindex_df)

    index_stock_cons_weight_csindex_df = index_stock_cons_weight_csindex(
        symbol="000300"
    )
    print(index_stock_cons_weight_csindex_df)

    index_stock_cons_sina_df = index_stock_cons_sina(symbol="000300")
    print(index_stock_cons_sina_df)

    index_stock_cons_df = index_stock_cons(symbol="000300")
    print(index_stock_cons_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/6/14 17:00
Desc: 东方财富网-数据中心-融资融券-融资融券账户统计-两融账户信息
https://www.szse.cn/disclosure/margin/object/index.html
"""

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def stock_margin_account_info() -> pd.DataFrame:
    """
    东方财富网-数据中心-融资融券-融资融券账户统计-两融账户信息
    https://data.eastmoney.com/rzrq/zhtjday.html
    :return: 融资融券账户统计
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(action="ignore", category=FutureWarning)
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPTA_WEB_MARGIN_DAILYTRADE",
        "columns": "ALL",
        "pageNumber": "1",
        "pageSize": "500",
        "sortColumns": "STATISTICS_DATE",
        "sortTypes": "-1",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url=url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    tqdm = get_tqdm()
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
                "p": page,
                "pageNo": page,
                "pageNum": page,
            }
        )
        r = requests.get(url=url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.rename(
        columns={
            "STATISTICS_DATE": "日期",
            "FIN_BALANCE": "融资余额",
            "LOAN_BALANCE": "融券余额",
            "FIN_BUY_AMT": "融资买入额",
            "LOAN_SELL_AMT": "融券卖出额",
            "SECURITY_ORG_NUM": "证券公司数量",
            "OPERATEDEPT_NUM": "营业部数量",
            "PERSONAL_INVESTOR_NUM": "个人投资者数量",
            "ORG_INVESTOR_NUM": "机构投资者数量",
            "INVESTOR_NUM": "参与交易的投资者数量",
            "MARGINLIAB_INVESTOR_NUM": "有融资融券负债的投资者数量",
            "TOTAL_GUARANTEE": "担保物总价值",
            "AVG_GUARANTEE_RATIO": "平均维持担保比例",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "日期",
            "融资余额",
            "融券余额",
            "融资买入额",
            "融券卖出额",
            "证券公司数量",
            "营业部数量",
            "个人投资者数量",
            "机构投资者数量",
            "参与交易的投资者数量",
            "有融资融券负债的投资者数量",
            "担保物总价值",
            "平均维持担保比例",
        ]
    ]

    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    for item in big_df.columns[1:]:
        big_df[item] = pd.to_numeric(big_df[item], errors="coerce")
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


if __name__ == "__main__":
    stock_margin_account_info_df = stock_margin_account_info()
    print(stock_margin_account_info_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/10/2 9:00
Desc: 股票基本面数据
新浪财经-财务报表-财务摘要
https://vip.stock.finance.sina.com.cn/corp/go.php/vFD_FinanceSummary/stockid/600004.phtml
新浪财经-财务分析-财务指标
https://money.finance.sina.com.cn/corp/go.php/vFD_FinancialGuideLine/stockid/600004/displaytype/4.phtml
新浪财经-发行与分配
https://money.finance.sina.com.cn/corp/go.php/vISSUE_ShareBonus/stockid/600004.phtml
"""

from datetime import datetime
from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup

from akshare.utils.tqdm import get_tqdm


def stock_financial_report_sina(
    stock: str = "sh600600", symbol: str = "资产负债表"
) -> pd.DataFrame:
    """
    新浪财经-财务报表-三大报表
    https://vip.stock.finance.sina.com.cn/corp/go.php/vFD_FinanceSummary/stockid/600600/displaytype/4.phtml?source=fzb&qq-pf-to=pcqq.group
    :param stock: 股票代码
    :type stock: str
    :param symbol: choice of {"资产负债表", "利润表", "现金流量表"}
    :type symbol: str
    :return: 新浪财经-财务报表-三大报表
    :rtype: pandas.DataFrame
    """
    symbol_map = {"资产负债表": "fzb", "利润表": "lrb", "现金流量表": "llb"}
    url = "https://quotes.sina.cn/cn/api/openapi.php/CompanyFinanceService.getFinanceReport2022"
    params = {
        "paperCode": f"{stock}",
        "source": symbol_map[symbol],
        "type": "0",
        "page": "1",
        "num": "1000",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    df_columns = [
        item["date_value"] for item in data_json["result"]["data"]["report_date"]
    ]
    big_df = pd.DataFrame()
    temp_df = pd.DataFrame()
    for date_str in df_columns:
        temp_df = pd.DataFrame(
            data_json["result"]["data"]["report_list"][date_str]["data"]
        )
        temp_df = temp_df[["item_title", "item_value"]]
        temp_df["item_value"] = pd.to_numeric(temp_df["item_value"], errors="coerce")
        temp_tail_df = pd.DataFrame.from_dict(
            data={
                "数据源": data_json["result"]["data"]["report_list"][date_str][
                    "data_source"
                ],
                "是否审计": data_json["result"]["data"]["report_list"][date_str][
                    "is_audit"
                ],
                "公告日期": data_json["result"]["data"]["report_list"][date_str][
                    "publish_date"
                ],
                "币种": data_json["result"]["data"]["report_list"][date_str][
                    "rCurrency"
                ],
                "类型": data_json["result"]["data"]["report_list"][date_str]["rType"],
                "更新日期": datetime.fromtimestamp(
                    data_json["result"]["data"]["report_list"][date_str]["update_time"]
                ).isoformat(),
            },
            orient="index",
        )
        temp_tail_df.reset_index(inplace=True)
        temp_tail_df.columns = ["item_title", "item_value"]
        temp_df = pd.concat(objs=[temp_df, temp_tail_df], ignore_index=True)
        temp_df.columns = ["项目", date_str]
        big_df = pd.concat(objs=[big_df, temp_df[date_str]], axis=1, ignore_index=True)

    big_df = big_df.T
    big_df.columns = temp_df["项目"]
    big_df = pd.concat(objs=[pd.DataFrame({"报告日": df_columns}), big_df], axis=1)
    # 此处有 '国内票证结算' 和 '内部应收款'字段重复
    big_df = big_df.loc[:, ~big_df.columns.duplicated(keep="first")]
    return big_df


def stock_financial_abstract(symbol: str = "600004") -> pd.DataFrame:
    """
    新浪财经-财务报表-关键指标
    https://vip.stock.finance.sina.com.cn/corp/go.php/vFD_FinanceSummary/stockid/600004.phtml
    :param symbol: 股票代码
    :type symbol: str
    :return: 新浪财经-财务报表-关键指标
    :rtype: pandas.DataFrame
    """
    url = "https://quotes.sina.cn/cn/api/openapi.php/CompanyFinanceService.getFinanceReport2022"
    params = {
        "paperCode": f"sh{symbol}",
        "source": "gjzb",
        "type": "0",
        "page": "1",
        "num": "1000",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    key_list = list(data_json["result"]["data"]["report_list"].keys())
    temp_df = pd.DataFrame(
        data_json["result"]["data"]["report_list"][key_list[0]]["data"]
    )
    big_df = temp_df["item_title"]
    for item in key_list:
        temp_df = pd.DataFrame(data_json["result"]["data"]["report_list"][item]["data"])
        big_df = pd.concat(
            objs=[big_df, temp_df["item_value"]], axis=1, ignore_index=True
        )
    big_df.index = big_df.iloc[:, 0]
    big_df = big_df.iloc[:, 1:]

    big_one_df = big_df.loc["常用指标":"每股指标"]
    big_one_df = big_one_df.iloc[1:-1, :]
    big_one_df.reset_index(inplace=True)
    big_one_df.insert(0, "选项", "常用指标")

    big_two_df = big_df.loc["每股指标":"盈利能力"]
    big_two_df = big_two_df.iloc[1:-1, :]
    big_two_df.reset_index(inplace=True)
    big_two_df.insert(0, "选项", "每股指标")

    big_three_df = big_df.loc["盈利能力":"成长能力"]
    big_three_df = big_three_df.iloc[1:-1, :]
    big_three_df.reset_index(inplace=True)
    big_three_df.insert(0, "选项", "盈利能力")

    big_four_df = big_df.loc["成长能力":"收益质量"]
    big_four_df = big_four_df.iloc[1:-1, :]
    big_four_df.reset_index(inplace=True)
    big_four_df.insert(0, "选项", "成长能力")

    big_five_df = big_df.loc["收益质量":"财务风险"]
    big_five_df = big_five_df.iloc[1:-1, :]
    big_five_df.reset_index(inplace=True)
    big_five_df.insert(0, "选项", "收益质量")

    big_six_df = big_df.loc["财务风险":"营运能力"]
    big_six_df = big_six_df.iloc[1:-1, :]
    big_six_df.reset_index(inplace=True)
    big_six_df.insert(0, "选项", "财务风险")

    big_seven_df = big_df.loc["营运能力":]
    big_seven_df = big_seven_df.iloc[1:-1, :]
    big_seven_df.reset_index(inplace=True)
    big_seven_df.insert(0, "选项", "营运能力")

    big_df = pd.concat(
        objs=[
            big_one_df,
            big_two_df,
            big_three_df,
            big_four_df,
            big_five_df,
            big_six_df,
            big_seven_df,
        ],
        ignore_index=True,
    )
    key_list.insert(0, "选项")
    key_list.insert(1, "指标")
    big_df.columns = key_list
    for item in big_df.columns[2:]:
        big_df[item] = pd.to_numeric(big_df[item], errors="coerce")
    return big_df


def stock_financial_analysis_indicator_em(
    symbol: str = "301389.SZ", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    东方财富-A股-财务分析-主要指标
    https://emweb.securities.eastmoney.com/pc_hsf10/pages/index.html?type=web&code=SZ301389&color=b#/cwfx
    :param symbol: 股票代码（带市场标识）
    :type symbol: str
    :param indicator: choice of {"按报告期", "按单季度"}
    :type indicator: str
    :return: 东方财富-A股-财务分析-主要指标
    :rtype: pandas.DataFrame
    """
    if indicator == "按报告期":
        url = "https://datacenter.eastmoney.com/securities/api/data/get"
        params = {
            "type": "RPT_F10_FINANCE_MAINFINADATA",
            "sty": "APP_F10_MAINFINADATA",
            "quoteColumns": "",
            "filter": f"""(SECUCODE="{symbol}")""",
            "p": "1",
            "ps": "200",
            "sr": "-1",
            "st": "REPORT_DATE",
            "source": "HSF10",
            "client": "PC",
        }
    else:
        url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
        params = {
            "reportName": "RPT_F10_QTR_MAINFINADATA",
            "columns": "ALL",
            "quoteColumns": "",
            "filter": f"""(SECUCODE="{symbol}")""",
            "pageNumber": "1",
            "pageSize": "200",
            "sortTypes": "-1",
            "sortColumns": "REPORT_DATE",
            "source": "HSF10",
            "client": "PC",
        }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    return temp_df


def stock_financial_analysis_indicator(
    symbol: str = "600004", start_year: str = "1900"
) -> pd.DataFrame:
    """
    新浪财经-财务分析-财务指标
    https://money.finance.sina.com.cn/corp/go.php/vFD_FinancialGuideLine/stockid/600004/ctrl/2019/displaytype/4.phtml
    :param symbol: 股票代码
    :type symbol: str
    :param start_year: 开始年份
    :type start_year: str
    :return: 新浪财经-财务分析-财务指标
    :rtype: pandas.DataFrame
    """
    url = (
        f"https://money.finance.sina.com.cn/corp/go.php/vFD_FinancialGuideLine/"
        f"stockid/{symbol}/ctrl/2020/displaytype/4.phtml"
    )
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    year_context = soup.find(attrs={"id": "con02-1"}).find("table").find_all("a")
    year_list = [item.text for item in year_context]
    if start_year in year_list:
        year_list = year_list[: year_list.index(start_year) + 1]
    else:
        return pd.DataFrame()
    out_df = pd.DataFrame()
    tqdm = get_tqdm()
    for year_item in tqdm(year_list, leave=False):
        url = (
            f"https://money.finance.sina.com.cn/corp/go.php/vFD_FinancialGuideLine/"
            f"stockid/{symbol}/ctrl/{year_item}/displaytype/4.phtml"
        )
        r = requests.get(url)
        temp_df = pd.read_html(StringIO(r.text))[12].iloc[:, :-1]
        temp_df.columns = temp_df.iloc[0, :]
        temp_df = temp_df.iloc[1:, :]
        big_df = pd.DataFrame()
        indicator_list = [
            "每股指标",
            "盈利能力",
            "成长能力",
            "营运能力",
            "偿债及资本结构",
            "现金流量",
            "其他指标",
        ]
        for i in range(len(indicator_list)):
            if i == 6:
                inner_df = temp_df[
                    temp_df.loc[
                        temp_df.iloc[:, 0].str.find(indicator_list[i]) == 0, :
                    ].index[0] :
                ].T
            else:
                inner_df = temp_df[
                    temp_df.loc[
                        temp_df.iloc[:, 0].str.find(indicator_list[i]) == 0, :
                    ].index[0] : temp_df.loc[
                        temp_df.iloc[:, 0].str.find(indicator_list[i + 1]) == 0, :
                    ].index[0]
                    - 1
                ].T
            inner_df = inner_df.reset_index(drop=True)
            big_df = pd.concat(objs=[big_df, inner_df], axis=1)
        big_df.columns = big_df.iloc[0, :].tolist()
        big_df = big_df.iloc[1:, :]
        big_df.index = temp_df.columns.tolist()[1:]
        out_df = pd.concat(objs=[out_df, big_df])

    out_df.dropna(inplace=True)
    out_df.reset_index(inplace=True)
    out_df.rename(columns={"index": "日期"}, inplace=True)
    out_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    out_df["日期"] = pd.to_datetime(out_df["日期"], errors="coerce").dt.date
    for item in out_df.columns[1:]:
        out_df[item] = pd.to_numeric(out_df[item], errors="coerce")
    return out_df


def stock_history_dividend() -> pd.DataFrame:
    """
    新浪财经-发行与分配-历史分红
    https://vip.stock.finance.sina.com.cn/q/go.php/vInvestConsult/kind/lsfh/index.phtml
    :return: 所有股票的历史分红数据
    :rtype: pandas.DataFrame
    """
    url = "https://vip.stock.finance.sina.com.cn/q/go.php/vInvestConsult/kind/lsfh/index.phtml"
    params = {"p": "1", "num": "50000"}
    r = requests.get(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df["代码"] = temp_df["代码"].astype(str).str.zfill(6)
    temp_df.columns = [
        "代码",
        "名称",
        "上市日期",
        "累计股息",
        "年均股息",
        "分红次数",
        "融资总额",
        "融资次数",
        "详细",
    ]
    temp_df["上市日期"] = pd.to_datetime(temp_df["上市日期"], errors="coerce").dt.date
    temp_df["累计股息"] = pd.to_numeric(temp_df["累计股息"], errors="coerce")
    temp_df["年均股息"] = pd.to_numeric(temp_df["年均股息"], errors="coerce")
    temp_df["分红次数"] = pd.to_numeric(temp_df["分红次数"], errors="coerce")
    temp_df["融资总额"] = pd.to_numeric(temp_df["融资总额"], errors="coerce")
    temp_df["融资次数"] = pd.to_numeric(temp_df["融资次数"], errors="coerce")
    del temp_df["详细"]
    return temp_df


def stock_history_dividend_detail(
    symbol: str = "000002", indicator: str = "分红", date: str = ""
) -> pd.DataFrame:
    """
    新浪财经-发行与分配-分红配股详情
    https://vip.stock.finance.sina.com.cn/corp/go.php/vISSUE_ShareBonus/stockid/300670.phtml
    :param indicator: choice of {"分红", "配股"}
    :type indicator: str
    :param symbol: 股票代码
    :type symbol: str
    :param date: 分红配股的具体日期, e.g., "1994-12-24"
    :type date: str
    :return: 指定 indicator, stock, date 的数据
    :rtype: pandas.DataFrame
    """
    if indicator == "分红":
        url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vISSUE_ShareBonus/stockid/{symbol}.phtml"
        r = requests.get(url)
        temp_df = pd.read_html(StringIO(r.text))[12]
        temp_df.columns = [item[2] for item in temp_df.columns.tolist()]
        temp_df.columns = [
            "公告日期",
            "送股",
            "转增",
            "派息",
            "进度",
            "除权除息日",
            "股权登记日",
            "红股上市日",
            "查看详细",
        ]
        del temp_df["查看详细"]
        if temp_df.iloc[0, 0] == "暂时没有数据！":
            return pd.DataFrame()
        temp_df["公告日期"] = pd.to_datetime(
            temp_df["公告日期"], errors="coerce"
        ).dt.date
        temp_df["送股"] = pd.to_numeric(temp_df["送股"], errors="coerce")
        temp_df["转增"] = pd.to_numeric(temp_df["转增"], errors="coerce")
        temp_df["派息"] = pd.to_numeric(temp_df["派息"], errors="coerce")
        temp_df["除权除息日"] = pd.to_datetime(
            temp_df["除权除息日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        temp_df["股权登记日"] = pd.to_datetime(
            temp_df["股权登记日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        temp_df["红股上市日"] = pd.to_datetime(
            temp_df["红股上市日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        if date:
            url = "https://vip.stock.finance.sina.com.cn/corp/view/vISSUE_ShareBonusDetail.php"
            params = {
                "stockid": symbol,
                "type": "1",
                "end_date": date,
            }
            r = requests.get(url, params=params)
            temp_df = pd.read_html(StringIO(r.text))[12]
            temp_df.columns = ["item", "value"]
            return temp_df
        else:
            return temp_df
    else:
        url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vISSUE_ShareBonus/stockid/{symbol}.phtml"
        r = requests.get(url)
        temp_df = pd.read_html(StringIO(r.text))[13]
        temp_df.columns = [item[1] for item in temp_df.columns.tolist()]
        temp_df.columns = [
            "公告日期",
            "配股方案",
            "配股价格",
            "基准股本",
            "除权日",
            "股权登记日",
            "缴款起始日",
            "缴款终止日",
            "配股上市日",
            "募集资金合计",
            "查看详细",
        ]
        del temp_df["查看详细"]
        if temp_df.iloc[0, 0] == "暂时没有数据！":
            return pd.DataFrame()
        temp_df["公告日期"] = pd.to_datetime(
            temp_df["公告日期"], errors="coerce"
        ).dt.date
        temp_df["配股方案"] = pd.to_numeric(temp_df["配股方案"], errors="coerce")
        temp_df["配股价格"] = pd.to_numeric(temp_df["配股价格"], errors="coerce")
        temp_df["基准股本"] = pd.to_numeric(temp_df["基准股本"], errors="coerce")
        temp_df["募集资金合计"] = pd.to_numeric(
            temp_df["募集资金合计"], errors="coerce"
        )
        temp_df["除权日"] = pd.to_datetime(
            temp_df["除权日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        temp_df["股权登记日"] = pd.to_datetime(
            temp_df["股权登记日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        temp_df["缴款起始日"] = pd.to_datetime(
            temp_df["缴款起始日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        temp_df["缴款终止日"] = pd.to_datetime(
            temp_df["缴款终止日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        temp_df["配股上市日"] = pd.to_datetime(
            temp_df["配股上市日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date

        if date:
            url = "https://vip.stock.finance.sina.com.cn/corp/view/vISSUE_ShareBonusDetail.php"
            params = {
                "stockid": symbol,
                "type": "1",
                "end_date": date,
            }
            r = requests.get(url, params=params)
            temp_df = pd.read_html(StringIO(r.text))[12]
            temp_df.columns = ["item", "value"]
            return temp_df
        else:
            return temp_df


def stock_ipo_info(stock: str = "600004") -> pd.DataFrame:
    """
    新浪财经-发行与分配-新股发行
    https://vip.stock.finance.sina.com.cn/corp/go.php/vISSUE_NewStock/stockid/600004.phtml
    :param stock: 股票代码
    :type stock: str
    :return: 返回新股发行详情
    :rtype: pandas.DataFrame
    """
    url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vISSUE_NewStock/stockid/{stock}.phtml"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[12]
    temp_df.columns = ["item", "value"]
    return temp_df


def stock_add_stock(symbol: str = "688166") -> pd.DataFrame:
    """
    新浪财经-发行与分配-增发
    https://vip.stock.finance.sina.com.cn/corp/go.php/vISSUE_AddStock/stockid/600004.phtml
    :param symbol: 股票代码
    :type symbol: str
    :return: 返回增发详情
    :rtype: pandas.DataFrame
    """
    url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vISSUE_AddStock/stockid/{symbol}.phtml"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[12]
    if temp_df.at[0, 0] == "对不起，暂时没有相关增发记录":
        raise f"股票 {symbol} 无增发记录"
    big_df = pd.DataFrame()
    for i in range(int(len(temp_df.at[0, 1]) / 10)):
        temp_df = pd.read_html(StringIO(r.text))[13 + i].iloc[:, 1]
        big_df[temp_df.name.split(" ")[1].split("：")[1][:10]] = temp_df
    big_df = big_df.T
    big_df.reset_index(inplace=True)
    big_df.columns = [
        "公告日期",
        "发行方式",
        "发行价格",
        "实际公司募集资金总额",
        "发行费用总额",
        "实际发行数量",
    ]
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


def stock_restricted_release_queue_sina(symbol: str = "600000") -> pd.DataFrame:
    """
    新浪财经-发行分配-限售解禁
    https://vip.stock.finance.sina.com.cn/q/go.php/vInvestConsult/kind/xsjj/index.phtml?symbol=sh600000
    :param symbol: 股票代码
    :type symbol: str
    :return: 返回限售解禁数据
    :rtype: pandas.DataFrame
    """
    url = f"https://vip.stock.finance.sina.com.cn/q/go.php/vInvestConsult/kind/xsjj/index.phtml?symbol={symbol}"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.columns = [
        "代码",
        "名称",
        "解禁日期",
        "解禁数量",
        "解禁股流通市值",
        "上市批次",
        "公告日期",
    ]
    temp_df["解禁日期"] = pd.to_datetime(temp_df["解禁日期"], errors="coerce").dt.date
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["解禁数量"] = pd.to_numeric(temp_df["解禁数量"], errors="coerce")
    temp_df["解禁股流通市值"] = pd.to_numeric(
        temp_df["解禁股流通市值"], errors="coerce"
    )
    temp_df["上市批次"] = pd.to_numeric(temp_df["上市批次"], errors="coerce")
    temp_df["上市批次"] = pd.to_numeric(temp_df["上市批次"], errors="coerce")
    return temp_df


def stock_circulate_stock_holder(symbol: str = "600000") -> pd.DataFrame:
    """
    新浪财经-股东股本-流通股东
    P.S. 特定股票特定时间只有前 5 个; e.g., 000002
    https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_CirculateStockHolder/stockid/600000.phtml
    :param symbol: 股票代码
    :type symbol: str
    :return: 新浪财经-股东股本-流通股东
    :rtype: pandas.DataFrame
    """
    pd.set_option("future.no_silent_downcasting", True)
    url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_CirculateStockHolder/stockid/{symbol}.phtml"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[13].iloc[:, :5]
    temp_df.columns = [*range(5)]
    big_df = pd.DataFrame()
    need_range = temp_df[
        temp_df.iloc[:, 0].str.find("截止日期") == 0
    ].index.tolist() + [len(temp_df)]
    tqdm = get_tqdm()
    for i in tqdm(range(len(need_range) - 1), leave=False):
        truncated_df = temp_df.iloc[need_range[i] : need_range[i + 1], :]
        truncated_df = truncated_df.dropna(how="all")
        temp_truncated = truncated_df.iloc[2:, :]
        temp_truncated.reset_index(inplace=True, drop=True)
        concat_df = pd.concat(
            objs=[temp_truncated, truncated_df.iloc[0, :], truncated_df.iloc[1, :]],
            axis=1,
        )
        concat_df.columns = concat_df.iloc[0, :]
        concat_df = concat_df.iloc[1:, :]
        try:
            # try for pandas >= 2.1.0
            concat_df["截止日期"] = concat_df["截止日期"].ffill()
            concat_df["公告日期"] = concat_df["公告日期"].ffill()
        except:  # noqa: E722
            try:
                # try for pandas < 2.1.0
                concat_df["截止日期"] = concat_df["截止日期"].fillna(method="ffill")
                concat_df["公告日期"] = concat_df["公告日期"].fillna(method="ffill")
            except Exception as e:
                print("Error:", e)

        big_df = pd.concat(objs=[big_df, concat_df], axis=0, ignore_index=True)

    big_df = big_df[
        [
            "截止日期",
            "公告日期",
            "编号",
            "股东名称",
            "持股数量(股)",
            "占流通股比例(%)",
            "股本性质",
        ]
    ]
    big_df.columns = [
        "截止日期",
        "公告日期",
        "编号",
        "股东名称",
        "持股数量",
        "占流通股比例",
        "股本性质",
    ]

    big_df["截止日期"] = pd.to_datetime(big_df["截止日期"], errors="coerce").dt.date
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    big_df["编号"] = pd.to_numeric(big_df["编号"], errors="coerce")
    big_df["持股数量"] = pd.to_numeric(big_df["持股数量"], errors="coerce")
    big_df["占流通股比例"] = pd.to_numeric(big_df["占流通股比例"], errors="coerce")
    big_df.reset_index(inplace=True, drop=True)
    return big_df


def stock_fund_stock_holder(symbol: str = "600004") -> pd.DataFrame:
    """
    新浪财经-股本股东-基金持股
    https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_FundStockHolder/stockid/600004.phtml
    :param symbol: 股票代码
    :type symbol: str
    :return: 新浪财经-股本股东-基金持股
    :rtype: pandas.DataFrame
    """
    url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_FundStockHolder/stockid/{symbol}.phtml"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[13].iloc[:, :6]
    temp_df.columns = [*range(6)]
    big_df = pd.DataFrame()
    need_range = temp_df[
        temp_df.iloc[:, 0].str.find("截止日期") == 0
    ].index.tolist() + [len(temp_df)]
    tqdm = get_tqdm()
    for i in tqdm(range(len(need_range) - 1), leave=False):
        # pass
        truncated_df = temp_df.iloc[need_range[i] : need_range[i + 1], :]
        truncated_df = truncated_df.dropna(how="all")
        temp_truncated = truncated_df.iloc[2:, :]
        temp_truncated.reset_index(inplace=True, drop=True)
        concat_df = pd.concat(objs=[temp_truncated, truncated_df.iloc[0, 1:]], axis=1)
        concat_df.columns = truncated_df.iloc[1, :].tolist() + ["截止日期"]
        try:
            # try for pandas >= 2.1.0
            concat_df["截止日期"] = concat_df["截止日期"].ffill()
            concat_df["截止日期"] = concat_df["截止日期"].ffill()
        except:  # noqa: E722
            try:
                # try for pandas < 2.1.0
                concat_df["截止日期"] = concat_df["截止日期"].fillna(method="ffill")
                concat_df["截止日期"] = concat_df["截止日期"].fillna(method="bfill")
            except Exception as e:
                print("Error:", e)

        big_df = pd.concat(objs=[big_df, concat_df], axis=0, ignore_index=True)
    big_df.dropna(inplace=True)
    big_df.reset_index(inplace=True, drop=True)
    big_df.columns = [
        "基金名称",
        "基金代码",
        "持仓数量",
        "占流通股比例",
        "持股市值",
        "占净值比例",
        "截止日期",
    ]
    big_df["持仓数量"] = pd.to_numeric(big_df["持仓数量"], errors="coerce")
    big_df["占流通股比例"] = pd.to_numeric(big_df["占流通股比例"], errors="coerce")
    big_df["持股市值"] = pd.to_numeric(big_df["持股市值"], errors="coerce")
    big_df["占净值比例"] = pd.to_numeric(big_df["占净值比例"], errors="coerce")
    big_df["截止日期"] = pd.to_datetime(big_df["截止日期"], errors="coerce").dt.date
    return big_df


def stock_main_stock_holder(stock: str = "600004") -> pd.DataFrame:
    """
    新浪财经-股本股东-主要股东
    P.S. 特定股票特定时间只有前 5 个; e.g., 000002
    https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_StockHolder/stockid/600004.phtml
    :param stock: 股票代码
    :type stock: str
    :return: 新浪财经-股本股东-主要股东
    :rtype: pandas.DataFrame
    """
    url = f"https://vip.stock.finance.sina.com.cn/corp/go.php/vCI_StockHolder/stockid/{stock}.phtml"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[13].iloc[:, :5]
    temp_df.columns = [*range(5)]
    big_df = pd.DataFrame()
    need_range = temp_df[
        temp_df.iloc[:, 0].str.find("截至日期") == 0
    ].index.tolist() + [len(temp_df)]
    for i in range(len(need_range) - 1):
        truncated_df = temp_df.iloc[need_range[i] : need_range[i + 1], :]
        truncated_df = truncated_df.dropna(how="all")
        temp_truncated = truncated_df.iloc[5:, :]
        temp_truncated.reset_index(inplace=True, drop=True)
        concat_df = pd.concat(
            objs=[
                temp_truncated,
                truncated_df.iloc[0, :],
                truncated_df.iloc[1, :],
                truncated_df.iloc[2, :],
                truncated_df.iloc[3, :],
                truncated_df.iloc[4, :],
            ],
            axis=1,
        )
        concat_df.columns = concat_df.iloc[0, :]
        concat_df = concat_df.iloc[1:, :].copy()
        try:
            # try for pandas >= 2.1.0
            concat_df["截至日期"] = concat_df["截至日期"].ffill()
            concat_df["公告日期"] = concat_df["公告日期"].ffill()
            concat_df["股东总数"] = concat_df["股东总数"].ffill()
            concat_df["平均持股数"] = concat_df["平均持股数"].ffill()
        except:  # noqa: E722
            try:
                # try for pandas < 2.1.0
                concat_df["截至日期"] = concat_df["截至日期"].fillna(method="ffill")
                concat_df["公告日期"] = concat_df["公告日期"].fillna(method="ffill")
                concat_df["股东总数"] = concat_df["股东总数"].fillna(method="ffill")
                concat_df["平均持股数"] = concat_df["平均持股数"].fillna(method="ffill")
            except Exception as e:
                print("Error:", e)

        concat_df["股东总数"] = concat_df["股东总数"].str.strip("查看变化趋势")
        concat_df["平均持股数"] = concat_df["平均持股数"].str.strip(
            "(按总股本计算) 查看变化趋势"
        )
        big_df = pd.concat(objs=[big_df, concat_df], axis=0, ignore_index=True)
    big_df.dropna(inplace=True, how="all")
    big_df.reset_index(inplace=True, drop=True)
    big_df.rename(
        columns={"持股数量(股)": "持股数量", "持股比例(%)": "持股比例"}, inplace=True
    )
    big_df.columns.name = None
    big_df["持股数量"] = pd.to_numeric(big_df["持股数量"], errors="coerce")
    big_df["持股比例"] = big_df["持股比例"].str.strip("↓")
    big_df["持股比例"] = pd.to_numeric(big_df["持股比例"], errors="coerce")
    big_df["截至日期"] = pd.to_datetime(big_df["截至日期"], errors="coerce").dt.date
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    big_df["股东总数"] = pd.to_numeric(big_df["股东总数"], errors="coerce")
    big_df["平均持股数"] = pd.to_numeric(big_df["平均持股数"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_financial_report_sina_df = stock_financial_report_sina(
        stock="sh600600", symbol="现金流量表"
    )
    print(stock_financial_report_sina_df)

    stock_financial_report_sina_df = stock_financial_report_sina(
        stock="sh600600", symbol="资产负债表"
    )
    print(stock_financial_report_sina_df)

    stock_financial_report_sina_df = stock_financial_report_sina(
        stock="sh600600", symbol="利润表"
    )
    print(stock_financial_report_sina_df)

    stock_financial_abstract_df = stock_financial_abstract(symbol="600600")
    print(stock_financial_abstract_df)

    stock_financial_analysis_indicator_df = stock_financial_analysis_indicator(
        symbol="600519", start_year="2019"
    )
    print(stock_financial_analysis_indicator_df)

    stock_history_dividend_df = stock_history_dividend()
    print(stock_history_dividend_df)

    stock_history_dividend_detail_df = stock_history_dividend_detail(
        symbol="600012", indicator="分红", date=""
    )
    print(stock_history_dividend_detail_df)

    stock_history_dividend_detail_df = stock_history_dividend_detail(
        symbol="600012", indicator="分红", date="2019-07-08"
    )
    print(stock_history_dividend_detail_df)

    stock_history_dividend_detail_df = stock_history_dividend_detail(
        symbol="000002", indicator="配股"
    )
    print(stock_history_dividend_detail_df)

    stock_history_dividend_detail_df = stock_history_dividend_detail(
        symbol="000002", indicator="配股", date="1999-12-22"
    )
    print(stock_history_dividend_detail_df)

    stock_ipo_info_df = stock_ipo_info(stock="600004")
    print(stock_ipo_info_df)

    stock_add_stock_df = stock_add_stock(symbol="600004")
    print(stock_add_stock_df)

    stock_restricted_release_queue_sina_df = stock_restricted_release_queue_sina(
        symbol="600000"
    )
    print(stock_restricted_release_queue_sina_df)

    stock_circulate_stock_holder_df = stock_circulate_stock_holder(symbol="600000")
    print(stock_circulate_stock_holder_df)

    stock_fund_stock_holder_df = stock_fund_stock_holder(symbol="601318")
    print(stock_fund_stock_holder_df)

    stock_main_stock_holder_df = stock_main_stock_holder(stock="600000")
    print(stock_main_stock_holder_df)

    stock_financial_analysis_indicator_em_df = stock_financial_analysis_indicator_em(symbol="301389.SZ", indicator="按报告期")
    print(stock_financial_analysis_indicator_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/1/9 22:00
Desc: 新浪财经-A股-实时行情数据和历史行情数据(包含前复权和后复权因子)
https://finance.sina.com.cn/realstock/company/sh689009/nc.shtml
"""

import json
import re

import pandas as pd
import py_mini_racer
import requests

from akshare.stock.cons import (
    zh_sina_a_stock_payload,
    zh_sina_a_stock_url,
    zh_sina_a_stock_count_url,
    zh_sina_a_stock_hist_url,
    hk_js_decode,
    zh_sina_a_stock_hfq_url,
    zh_sina_a_stock_qfq_url,
    zh_sina_a_stock_amount_url
)
from akshare.utils import demjson
from akshare.utils.tqdm import get_tqdm


def _get_zh_a_page_count() -> int:
    """
    所有股票的总页数
    https://vip.stock.finance.sina.com.cn/mkt/#hs_a
    :return: 需要采集的股票总页数
    :rtype: int
    """
    res = requests.get(zh_sina_a_stock_count_url)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def stock_zh_a_spot() -> pd.DataFrame:
    """
    新浪财经-所有 A 股的实时行情数据; 重复运行本函数会被新浪暂时封 IP
    https://vip.stock.finance.sina.com.cn/mkt/#hs_a
    :return: 所有股票的实时行情数据
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = _get_zh_a_page_count()
    zh_sina_stock_payload_copy = zh_sina_a_stock_payload.copy()
    tqdm = get_tqdm()
    for page in tqdm(
        range(1, page_count + 1), leave=False, desc="Please wait for a moment"
    ):
        zh_sina_stock_payload_copy.update({"page": page})
        r = requests.get(zh_sina_a_stock_url, params=zh_sina_stock_payload_copy)
        data_json = demjson.decode(r.text)
        big_df = pd.concat(objs=[big_df, pd.DataFrame(data_json)], ignore_index=True)

    big_df = big_df.astype(
        {
            "trade": "float",
            "pricechange": "float",
            "changepercent": "float",
            "buy": "float",
            "sell": "float",
            "settlement": "float",
            "open": "float",
            "high": "float",
            "low": "float",
            "volume": "float",
            "amount": "float",
            "per": "float",
            "pb": "float",
            "mktcap": "float",
            "nmc": "float",
            "turnoverratio": "float",
        }
    )
    big_df.columns = [
        "代码",
        "_",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "买入",
        "卖出",
        "昨收",
        "今开",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "时间戳",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "买入",
            "卖出",
            "昨收",
            "今开",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "时间戳",
        ]
    ]
    return big_df


def stock_zh_a_daily(
    symbol: str = "sh603843",
    start_date: str = "19900101",
    end_date: str = "21000118",
    adjust: str = "",
) -> pd.DataFrame:
    """
    新浪财经-A 股-个股的历史行情数据, 大量抓取容易封 IP
    https://finance.sina.com.cn/realstock/company/sh603843/nc.shtml
    :param symbol: sh600000
    :type symbol: str
    :param start_date: 20201103; 开始日期
    :type start_date: str
    :param end_date: 20201103; 结束日期
    :type end_date: str
    :param adjust: 默认为空: 返回不复权的数据; qfq: 返回前复权后的数据; hfq: 返回后复权后的数据; hfq-factor: 返回后复权因子; qfq-factor: 返回前复权因子
    :type adjust: str
    :return: 行情数据
    :rtype: pandas.DataFrame
    """

    def _fq_factor(method: str) -> pd.DataFrame:
        if method == "hfq":
            r = requests.get(zh_sina_a_stock_hfq_url.format(symbol))
            hfq_factor_df = pd.DataFrame(
                eval(r.text.split("=")[1].split("\n")[0])["data"]
            )
            if hfq_factor_df.shape[0] == 0:
                raise ValueError("sina hfq factor not available")
            hfq_factor_df.columns = ["date", "hfq_factor"]
            hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)
            del hfq_factor_df["date"]
            hfq_factor_df.reset_index(inplace=True)
            return hfq_factor_df
        else:
            r = requests.get(zh_sina_a_stock_qfq_url.format(symbol))
            qfq_factor_df = pd.DataFrame(
                eval(r.text.split("=")[1].split("\n")[0])["data"]
            )
            if qfq_factor_df.shape[0] == 0:
                raise ValueError("sina hfq factor not available")
            qfq_factor_df.columns = ["date", "qfq_factor"]
            qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
            del qfq_factor_df["date"]
            qfq_factor_df.reset_index(inplace=True)
            return qfq_factor_df

    if adjust in ("hfq-factor", "qfq-factor"):
        return _fq_factor(adjust.split("-")[0])

    r = requests.get(zh_sina_a_stock_hist_url.format(symbol))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", r.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    data_df = pd.DataFrame(dict_list)
    data_df.index = pd.to_datetime(data_df["date"]).dt.date
    del data_df["date"]
    try:
        del data_df["prevclose"]
    except:  # noqa: E722
        pass
    try:
        del data_df["postVol"]
        del data_df["postAmt"]
    except:  # noqa: E722
        pass
    data_df = data_df.astype("float")
    r = requests.get(zh_sina_a_stock_amount_url.format(symbol, symbol))
    amount_data_json = demjson.decode(r.text[r.text.find("["): r.text.rfind("]") + 1])
    amount_data_df = pd.DataFrame(amount_data_json)
    amount_data_df.columns = ["date", "outstanding_share"]
    amount_data_df.index = pd.to_datetime(amount_data_df.date)
    del amount_data_df["date"]
    temp_df = pd.merge(
        data_df, amount_data_df, left_index=True, right_index=True, how="outer"
    )
    try:
        # try for pandas >= 2.1.0
        temp_df.ffill(inplace=True)
    except Exception:
        try:
            temp_df.fillna(method="ffill", inplace=True)
        except Exception as e:
            print("Error:", e)

    temp_df = temp_df.astype(float)
    temp_df["outstanding_share"] = temp_df["outstanding_share"] * 10000
    temp_df["turnover"] = temp_df["volume"] / temp_df["outstanding_share"]
    temp_df.columns = [
        "open",
        "high",
        "low",
        "close",
        "volume",
        "amount",
        "outstanding_share",
        "turnover",
    ]
    if adjust == "":
        temp_df = temp_df[start_date:end_date]
        temp_df.drop_duplicates(
            subset=["open", "high", "low", "close", "volume", "amount"], inplace=True
        )
        temp_df["open"] = round(temp_df["open"], 2)
        temp_df["high"] = round(temp_df["high"], 2)
        temp_df["low"] = round(temp_df["low"], 2)
        temp_df["close"] = round(temp_df["close"], 2)
        temp_df.dropna(inplace=True)
        temp_df.drop_duplicates(inplace=True)
        temp_df.reset_index(inplace=True)
        temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
        return temp_df
    if adjust == "hfq":
        res = requests.get(zh_sina_a_stock_hfq_url.format(symbol))
        hfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])["data"]
        )
        hfq_factor_df.columns = ["date", "hfq_factor"]
        hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)
        del hfq_factor_df["date"]
        temp_df = pd.merge(
            temp_df,
            hfq_factor_df,
            left_index=True,
            right_index=True,
            how="outer",
        )
        try:
            # try for pandas >= 2.1.0
            temp_df.ffill(inplace=True)
        except Exception:
            try:
                # try for pandas < 2.1.0
                temp_df.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        temp_df = temp_df.astype(float)
        temp_df.dropna(inplace=True)
        temp_df.drop_duplicates(
            subset=["open", "high", "low", "close", "volume", "amount"], inplace=True
        )
        temp_df["open"] = temp_df["open"] * temp_df["hfq_factor"]
        temp_df["high"] = temp_df["high"] * temp_df["hfq_factor"]
        temp_df["close"] = temp_df["close"] * temp_df["hfq_factor"]
        temp_df["low"] = temp_df["low"] * temp_df["hfq_factor"]
        temp_df = temp_df.iloc[:, :-1]
        temp_df = temp_df[start_date:end_date]
        temp_df["open"] = round(temp_df["open"], 2)
        temp_df["high"] = round(temp_df["high"], 2)
        temp_df["low"] = round(temp_df["low"], 2)
        temp_df["close"] = round(temp_df["close"], 2)
        temp_df.dropna(inplace=True)
        temp_df.reset_index(inplace=True)
        temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
        return temp_df

    if adjust == "qfq":
        res = requests.get(zh_sina_a_stock_qfq_url.format(symbol))
        qfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])["data"]
        )
        qfq_factor_df.columns = ["date", "qfq_factor"]
        qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
        del qfq_factor_df["date"]
        temp_df = pd.merge(
            temp_df,
            qfq_factor_df,
            left_index=True,
            right_index=True,
            how="outer",
        )
        try:
            # try for pandas >= 2.1.0
            temp_df.ffill(inplace=True)
        except Exception:
            try:
                # try for pandas < 2.1.0
                temp_df.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        temp_df = temp_df.astype(float)
        temp_df.dropna(inplace=True)
        temp_df.drop_duplicates(
            subset=["open", "high", "low", "close", "volume", "amount"], inplace=True
        )
        temp_df["open"] = temp_df["open"] / temp_df["qfq_factor"]
        temp_df["high"] = temp_df["high"] / temp_df["qfq_factor"]
        temp_df["close"] = temp_df["close"] / temp_df["qfq_factor"]
        temp_df["low"] = temp_df["low"] / temp_df["qfq_factor"]
        temp_df = temp_df.iloc[:, :-1]
        temp_df = temp_df[start_date:end_date]
        temp_df["open"] = round(temp_df["open"], 2)
        temp_df["high"] = round(temp_df["high"], 2)
        temp_df["low"] = round(temp_df["low"], 2)
        temp_df["close"] = round(temp_df["close"], 2)
        temp_df.dropna(inplace=True)
        temp_df.reset_index(inplace=True)
        temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
        return temp_df


def stock_zh_a_cdr_daily(
    symbol: str = "sh689009",
    start_date: str = "19900101",
    end_date: str = "22201116",
) -> pd.DataFrame:
    """
    新浪财经-A股-CDR个股的历史行情数据, 大量抓取容易封 IP
    https://finance.sina.com.cn/realstock/company/sh689009/nc.shtml
    :param start_date: 20201103; 开始日期
    :type start_date: str
    :param end_date: 20201103; 结束日期
    :type end_date: str
    :param symbol: sh689009
    :type symbol: str
    :return: specific data
    :rtype: pandas.DataFrame
    """
    res = requests.get(zh_sina_a_stock_hist_url.format(symbol))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", res.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    data_df = pd.DataFrame(dict_list)
    data_df.index = pd.to_datetime(data_df["date"])
    del data_df["date"]
    data_df = data_df.astype("float")
    temp_df = data_df[start_date:end_date].copy()
    temp_df["open"] = pd.to_numeric(temp_df["open"])
    temp_df["high"] = pd.to_numeric(temp_df["high"])
    temp_df["low"] = pd.to_numeric(temp_df["low"])
    temp_df["close"] = pd.to_numeric(temp_df["close"])
    temp_df.reset_index(inplace=True)
    temp_df["date"] = temp_df["date"].dt.date
    return temp_df


def stock_zh_a_minute(
    symbol: str = "sh600519", period: str = "1", adjust: str = ""
) -> pd.DataFrame:
    """
    股票及股票指数历史行情数据-分钟数据
    https://finance.sina.com.cn/realstock/company/sh600519/nc.shtml
    :param symbol: sh000300
    :type symbol: str
    :param period: 1, 5, 15, 30, 60 分钟的数据
    :type period: str
    :param adjust: 默认为空: 返回不复权的数据; qfq: 返回前复权后的数据; hfq: 返回后复权后的数据;
    :type adjust: str
    :return: specific data
    :rtype: pandas.DataFrame
    """
    url = (
        "https://quotes.sina.cn/cn/api/jsonp_v2.php/=/CN_MarketDataService.getKLineData"
    )
    params = {
        "symbol": symbol,
        "scale": period,
        "ma": "no",
        "datalen": "1970",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    try:
        data_json = json.loads(data_text.split("=(")[1].split(");")[0])
        temp_df = pd.DataFrame(data_json).iloc[:, :6]
    except:  # noqa: E722
        url = f"https://quotes.sina.cn/cn/api/jsonp_v2.php/var%20_{symbol}_{period}_1658852984203=/CN_MarketDataService.getKLineData"
        params = {
            "symbol": symbol,
            "scale": period,
            "ma": "no",
            "datalen": "1970",
        }
        r = requests.get(url, params=params)
        data_text = r.text
        data_json = json.loads(data_text.split("=(")[1].split(");")[0])
        temp_df = pd.DataFrame(data_json).iloc[:, :6]
    if temp_df.empty:
        print(f"{symbol} 股票数据不存在，请检查是否已退市")
        return pd.DataFrame()
    try:
        stock_zh_a_daily(symbol=symbol, adjust="qfq")
    except:  # noqa: E722
        return temp_df

    if adjust == "":
        return temp_df

    if adjust == "qfq":
        temp_df[["date", "time"]] = temp_df["day"].str.split(" ", expand=True)
        # 处理没有最后一分钟的情况
        need_df = temp_df[
            [
                True if "09:31:00" <= item <= "15:00:00" else False
                for item in temp_df["time"]
            ]
        ]
        need_df.drop_duplicates(subset=["date"], keep="last", inplace=True)
        need_df.index = pd.to_datetime(need_df["date"])
        stock_zh_a_daily_qfq_df = stock_zh_a_daily(symbol=symbol, adjust="qfq")
        stock_zh_a_daily_qfq_df.index = pd.to_datetime(stock_zh_a_daily_qfq_df["date"])
        result_df = stock_zh_a_daily_qfq_df.iloc[-len(need_df):, :]["close"].astype(
            float
        ) / need_df["close"].astype(float)
        temp_df.index = pd.to_datetime(temp_df["date"])
        merged_df = pd.merge(temp_df, result_df, left_index=True, right_index=True)
        merged_df["open"] = merged_df["open"].astype(float) * merged_df["close_y"]
        merged_df["high"] = merged_df["high"].astype(float) * merged_df["close_y"]
        merged_df["low"] = merged_df["low"].astype(float) * merged_df["close_y"]
        merged_df["close"] = merged_df["close_x"].astype(float) * merged_df["close_y"]
        temp_df = merged_df[["day", "open", "high", "low", "close", "volume"]]
        temp_df.reset_index(drop=True, inplace=True)
        return temp_df
    if adjust == "hfq":
        temp_df[["date", "time"]] = temp_df["day"].str.split(" ", expand=True)
        # 处理没有最后一分钟的情况
        need_df = temp_df[
            [
                True if "09:31:00" <= item <= "15:00:00" else False
                for item in temp_df["time"]
            ]
        ]
        need_df.drop_duplicates(subset=["date"], keep="last", inplace=True)
        need_df.index = pd.to_datetime(need_df["date"])
        stock_zh_a_daily_hfq_df = stock_zh_a_daily(symbol=symbol, adjust="hfq")
        stock_zh_a_daily_hfq_df.index = pd.to_datetime(stock_zh_a_daily_hfq_df["date"])
        result_df = stock_zh_a_daily_hfq_df.iloc[-len(need_df):, :]["close"].astype(
            float
        ) / need_df["close"].astype(float)
        temp_df.index = pd.to_datetime(temp_df["date"])
        merged_df = pd.merge(temp_df, result_df, left_index=True, right_index=True)
        merged_df["open"] = merged_df["open"].astype(float) * merged_df["close_y"]
        merged_df["high"] = merged_df["high"].astype(float) * merged_df["close_y"]
        merged_df["low"] = merged_df["low"].astype(float) * merged_df["close_y"]
        merged_df["close"] = merged_df["close_x"].astype(float) * merged_df["close_y"]
        temp_df = merged_df[["day", "open", "high", "low", "close", "volume"]]
        temp_df.reset_index(drop=True, inplace=True)
        return temp_df
    else:
        return pd.DataFrame()


if __name__ == "__main__":
    stock_zh_a_daily_hfq_df_one = stock_zh_a_daily(
        symbol="sz000002",
        start_date="19910403",
        end_date="20260109",
        adjust="hfq",
    )
    print(stock_zh_a_daily_hfq_df_one)

    stock_zh_a_daily_hfq_df_three = stock_zh_a_daily(
        symbol="sz000001",
        start_date="19900103",
        end_date="20210118",
        adjust="qfq",
    )
    print(stock_zh_a_daily_hfq_df_three)

    stock_zh_a_daily_hfq_df_two = stock_zh_a_daily(
        symbol="sh000001", start_date="20101103", end_date="20210510"
    )
    print(stock_zh_a_daily_hfq_df_two)

    qfq_factor_df = stock_zh_a_daily(symbol="sz000002", adjust="qfq-factor")
    print(qfq_factor_df)

    hfq_factor_df = stock_zh_a_daily(symbol="sz000002", adjust="hfq-factor")
    print(hfq_factor_df)

    stock_zh_a_daily_hfq_factor_df = stock_zh_a_daily(
        symbol="sz000002", adjust="hfq-factor"
    )
    print(stock_zh_a_daily_hfq_factor_df)

    stock_zh_a_daily_df = stock_zh_a_daily(
        symbol="sz300798", start_date="20200601", end_date="20231101", adjust="hfq"
    )
    print(stock_zh_a_daily_df)

    stock_zh_a_cdr_daily_df = stock_zh_a_cdr_daily(
        symbol="sh689009", start_date="20201103", end_date="20201116"
    )
    print(stock_zh_a_cdr_daily_df)

    stock_zh_a_spot_df = stock_zh_a_spot()
    print(stock_zh_a_spot_df)

    stock_zh_a_minute_df = stock_zh_a_minute(
        symbol="sz000876", period="1", adjust="qfq"
    )
    print(stock_zh_a_minute_df)

    stock_zh_a_minute_df = stock_zh_a_minute(
        symbol="sh600519", period="1", adjust="hfq"
    )
    print(stock_zh_a_minute_df)

    stock_zh_a_cdr_daily_df = stock_zh_a_cdr_daily(
        symbol="sh689009", start_date="19900101", end_date="22201116"
    )
    print(stock_zh_a_cdr_daily_df)
